------------------------------------------------------------------------
-- Add the new columns
------------------------------------------------------------------------
alter table alerts.status add NcEventTimeStamp time;
go
alter table alerts.status add NcEventSecondCounter int;
go
alter table alerts.problem_events add NcEventSecondCounter int;
go
alter table alerts.problem_events add NcEventTimeStamp time;
go
alter table alerts.status add NcReporting int;
go


------------------------------------------------------------------------
-- Sub Second trigger group : nc_sub_second
------------------------------------------------------------------------
create or replace trigger group nc_sub_second;
go
alter trigger group nc_sub_second set enabled false;
go
------------------------------------------------------------------------
-- Historical Reporting trigger group : nc_historical_reporting
------------------------------------------------------------------------
create or replace trigger group nc_historical_reporting;
go
alter trigger group nc_historical_reporting set enabled false;
go

------------------------------------------------------------------------
-- Create custom database & table : subsecond.counter
------------------------------------------------------------------------
create database subsecond;
go

create table subsecond.counter persistent
(
        TargetTable		varchar(64) primary key,
        Counter       		int,
        Timestamp		time
);
go
insert into subsecond.counter values ('alerts.status',1,1289831823);
insert into subsecond.counter values ('alerts.journal',1,1289831823);
go

------------------------------------------------------------------------
-- Event trigger : alerts.status : nc_status_inserts
------------------------------------------------------------------------
create or replace trigger nc_status_reinserts
group nc_sub_second
priority 20
comment 'Counts new status table reinserts for sub-second events'
after reinsert on alerts.status
for each row
declare
time_then time;
time_now time;
subsec  int;
begin

	set time_now = getdate();

	for each row status_count in subsecond.counter where status_count.TargetTable = 'alerts.status'
	begin
		set time_then = status_count.Timestamp;
		set subsec = status_count.Counter;
		if ( time_now > time_then )
		then
			update subsecond.counter set Counter = 1, Timestamp = time_now where TargetTable = 'alerts.status';
                else
			update subsecond.counter set Counter = to_int(subsec) + 1 where TargetTable = 'alerts.status';
		end if;
	end;   
end;
go
------------------------------------------------------------------------
-- Event trigger : alerts.status : nc_status_inserts
------------------------------------------------------------------------
create or replace trigger nc_status_inserts
group nc_sub_second
priority 20
comment 'Counts new status table inserts for sub-second events'
after insert on alerts.status
for each row
declare
time_then time;
time_now time;
subsec  int;
begin

	set time_now = getdate();

	for each row status_count in subsecond.counter where status_count.TargetTable = 'alerts.status'
	begin
		set time_then = status_count.Timestamp;
		set subsec = status_count.Counter;
		if ( time_now > time_then )
		then
			update subsecond.counter set Counter = 1, Timestamp = time_now where TargetTable = 'alerts.status';
                else
			update subsecond.counter set Counter = to_int(subsec) + 1 where TargetTable = 'alerts.status';
		end if;
	end;   
end;
go

------------------------------------------------------------------------
-- Event trigger : alerts.status : nc_status_updates
------------------------------------------------------------------------
create or replace trigger nc_status_updates
group nc_sub_second
priority 20
comment 'Counts new status table updates for sub-second events'
after update on alerts.status
for each row
declare
time_then time;
time_now time;
subsec  int;
begin

	set time_now = getdate();

	for each row status_count in subsecond.counter where status_count.TargetTable = 'alerts.status'
	begin
		set time_then = status_count.Timestamp;
		set subsec = status_count.Counter;
		if ( time_now > time_then )
		then
			update subsecond.counter set Counter = 1, Timestamp = time_now where TargetTable = 'alerts.status';
                else
			update subsecond.counter set Counter = to_int(subsec) + 1 where TargetTable = 'alerts.status';
		end if;
	end;   
end;
go



------------------------------------------------------------------------
-- Journal trigger : alerts.journal : nc_journal_inserts
------------------------------------------------------------------------
create or replace trigger nc_journal_inserts
group nc_sub_second
priority 20
comment 'Counts new journal table inserts for sub-second journals'
after insert on alerts.journal
for each row
declare
time_then time;
time_now time;
subsec  int;
begin

	set time_now = getdate();

	for each row journal_count in subsecond.counter where journal_count.TargetTable = 'alerts.journal'
	begin
		set time_then = journal_count.Timestamp;
		set subsec = journal_count.Counter;
		if ( time_now > time_then )
		then
			update subsecond.counter set Counter = 1, Timestamp = time_now where TargetTable = 'alerts.journal';
                else
			update subsecond.counter set Counter = to_int(subsec) + 1 where TargetTable = 'alerts.journal';
		end if;
	end;   
end;
go



------------------------------------------------------------------------
-- Event trigger : alerts.status : nc_new_row
------------------------------------------------------------------------
create or replace trigger nc_new_row
group nc_sub_second
enabled true 
priority 1
comment 'nc_new_row - added to allow event management'
before insert on alerts.status
for each row
begin

-- For events created in this object server
	if ( %user.is_gateway = false ) 
	then
              set new.Tally = 1;
	      set new.ServerName = getservername();
	end if;

-- Set NcEventSecondCounter
	for each row status_count in subsecond.counter where status_count.TargetTable = 'alerts.status'
	begin
		set new.NcEventSecondCounter = status_count.Counter;
	end;

-- Set other fields
	set new.StateChange = getdate();
	set new.InternalLast = getdate();

	if( new.ServerSerial = 0 )
	then
		set new.ServerSerial = new.Serial;
	end if;
end;
go

------------------------------------------------------------------------
-- Event trigger : alerts.status : nc_deduplication
------------------------------------------------------------------------
create or replace trigger nc_deduplication
group nc_sub_second
enabled true 
priority 1
comment 'nc_deduplication - added for event management'
before reinsert on alerts.status
for each row
begin
	set old.Tally = old.Tally + 1;
	set old.LastOccurrence = new.LastOccurrence;
	set old.StateChange = getdate();
	set old.InternalLast = getdate();
	set old.Summary = new.Summary;
	set old.AlertKey = new.AlertKey;
	if (( old.Severity = 0) and (new.Severity > 0))
	then
		set old.Severity = new.Severity;
	end if;

-- Set Event timestamp
	set old.NcEventTimeStamp = getdate();

-- Set NcEventSecondCounter
	for each row status_count in subsecond.counter where status_count.TargetTable = 'alerts.status'
	begin
		set old.NcEventSecondCounter = status_count.Counter;
	end;

-- Set NcReporting
	set old.NcReporting = new.NcReporting;

-- Common configurations to maintain dual-resilient object servers
	if ( %user.is_gateway = true ) 
	then
		set old.Acknowledged = new.Acknowledged;
		set old.Flash = new.Flash;
		set old.OwnerUID = new.OwnerUID;
		set old.OwnerGID = new.OwnerGID;
		set old.Severity = new.Severity;
		set old.Tally = new.Tally;
	end if;
end;
go

------------------------------------------------------------------------
-- Event trigger : alerts.status : nc_generic_clear
------------------------------------------------------------------------
create or replace trigger nc_generic_clear
group nc_sub_second 
enabled true 
priority 1
comment 'Generic Clear with sub-second counter - Problem/Resolution'
every 7 seconds 
begin
	-- Populate a table with Type 1 events corresponding to any uncleared Type 2 events
	for each row problem in alerts.status where
				problem.Type = 1 and problem.Severity > 0 and
                                (problem.Node + problem.AlertKey + problem.AlertGroup + problem.Manager) in
                                ( select Node + AlertKey + AlertGroup + Manager from alerts.status where Severity > 0 and Type = 2 ) 
	begin
		insert into alerts.problem_events 
		(Identifier,NcEventTimeStamp,AlertKey,AlertGroup,Node,Manager,Resolved,NcEventSecondCounter) 
		values ( problem.Identifier, problem.NcEventTimeStamp, problem.AlertKey, problem.AlertGroup, 
		problem.Node, problem.Manager, false, problem.NcEventSecondCounter );
	end;

	-- For each resolution event, mark the corresponding problem_events entry as resolved
	-- and clear the resolution
	for each row resolution in alerts.status where resolution.Type = 2 and resolution.Severity > 0
	begin
-- Update problem event to Resolved
		update alerts.problem_events set Resolved = true where 
				(NcEventTimeStamp < resolution.NcEventTimeStamp and 
				Manager = resolution.Manager and Node = resolution.Node and 
				AlertKey = resolution.AlertKey and AlertGroup = resolution.AlertGroup) or 
				(NcEventTimeStamp = resolution.NcEventTimeStamp and 
				NcEventSecondCounter < resolution.NcEventSecondCounter and
				Manager = resolution.Manager and Node = resolution.Node and 
				AlertKey = resolution.AlertKey and AlertGroup = resolution.AlertGroup);
-- Set resolution Severity to cleared
		set resolution.Severity = 0;

	end;

	-- Clear the Resolved events
	for each row problem in alerts.problem_events where problem.Resolved = true
	begin
		update alerts.status via problem.Identifier set Severity = 0;	
	end;

	-- Remove all entries from the problems table
	delete from alerts.problem_events;
end;
go


------------------------------------------------------------------------
-- Journal insert procedure : nc_jinsert
------------------------------------------------------------------------
create procedure nc_jinsert
(
in serial int,
in uid int,
in tstamp utc,
in msg char(4080)
)
declare
subsec	int;
key_field	char(64);
begin
	for each row journal_count in subsecond.counter where journal_count.TargetTable = 'alerts.journal'
	begin
		set subsec = journal_count.Counter;
	end;   

set key_field = to_char(serial) + ':' + to_char(subsec) + ':' + to_char( uid ) + ':' + to_char(tstamp);

        insert into alerts.journal values (
		key_field,
                serial,
                uid,
                tstamp,
                split_multibyte(msg, 1, 255),
                split_multibyte(msg, 2, 255),
                split_multibyte(msg, 3, 255),
                split_multibyte(msg, 4, 255),
                split_multibyte(msg, 5, 255),
                split_multibyte(msg, 6, 255),
                split_multibyte(msg, 7, 255),
                split_multibyte(msg, 8, 255),
                split_multibyte(msg, 9, 255),
                split_multibyte(msg, 10, 255),
                split_multibyte(msg, 11, 255),
                split_multibyte(msg, 12, 255),
                split_multibyte(msg, 13, 255),
                split_multibyte(msg, 14, 255),
                split_multibyte(msg, 15, 255),
                split_multibyte(msg, 16, 255)
        );
end;
go

------------------------------------------------------------------------
-- Historical Reporting trigger: nc_delete_clears
------------------------------------------------------------------------
create or replace trigger nc_delete_clears
group nc_historical_reporting
priority 1
comment 'Delete events only after they have been processed'
every 60 seconds
begin
        delete from alerts.status where Severity = 0 and (NcReporting = 0 or NcReporting =2) and StateChange < (getdate() - 120);

end;
go

-- EOF
